﻿/********************************************************************************//*!
 * @file  sample_app_setting.h
 * @brief  Initial setting of the HPS and the Board.
 * 
 * @details  Initial setting of the HPS and the Board.
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2015/03/02  ---  #18042  新規作成.
 * @date 2015/06/25  ---  #20263  v15.0対応/文字コード変更(UTF-8) 他. 
 * @date 2016/07/26  ---  -----  v16.0での確認およびHelio以外の評価ボードへの対応を実施.
 * 
 * @attention 
 * Copyright (C) 2013-2016 ALTIMA Corp./ELSENA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

#ifndef __SAMPLE_APP_SETTING_H__
#define __SAMPLE_APP_SETTING_H__

/***********************************************************************************
 *  includes
 ***********************************************************************************/
#include <stdint.h>
#include "hwlib.h"
#include "alt_generalpurpose_io.h"

/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  proto types 
 ***********************************************************************************/

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
// NOTE: To enable debugging output, delete the next line and uncomment the
//   line after.
#define dprintf(...)
// #define dprintf  printf

/* Basic definition*/
#define	__WARN_UNUSED__		__attribute__((unused))

#define	_KB		1024UL
#define	_MB		(1024UL*1024UL)
#define	_GB		(1024UL*1024UL*1024UL)

typedef void (*CALLBACK_FUNKt)(uint32_t, void*);

#define ACP_WINDOW_BASE	(0x80000000)

/*** Definition for on-board-switch detection ***/
#define SAMPLE_SWITCH_BIT_SLIDE0	(0x00000001)
#define SAMPLE_SWITCH_BIT_SLIDE1	(0x00000002)
#define SAMPLE_SWITCH_BIT_SLIDE2	(0x00000004)
#define SAMPLE_SWITCH_BIT_SLIDE3	(0x00000008)
#define SAMPLE_SWITCH_BIT_SLIDEALL	(SAMPLE_SWITCH_BIT_SLIDE0 | SAMPLE_SWITCH_BIT_SLIDE1 | SAMPLE_SWITCH_BIT_SLIDE2 | SAMPLE_SWITCH_BIT_SLIDE3)
#define SAMPLE_SWITCH_BIT_PUSH0		(0x00000010)
#define SAMPLE_SWITCH_BIT_PUSH1		(0x00000020)
#define SAMPLE_SWITCH_BIT_PUSH2		(0x00000040)
#define SAMPLE_SWITCH_BIT_PUSH3		(0x00000080)
#define SAMPLE_SWITCH_BIT_PUSHALL	(SAMPLE_SWITCH_BIT_PUSH0 | SAMPLE_SWITCH_BIT_PUSH1 | SAMPLE_SWITCH_BIT_PUSH2 | SAMPLE_SWITCH_BIT_PUSH3)
#define SAMPLE_SWITCH_BIT_ALL		(SAMPLE_SWITCH_BIT_SLIDEALL | SAMPLE_SWITCH_BIT_PUSHALL)

#define SAMPLE_SWITCH_BIT_NUM		(SAMPLE_SWITCH_BIT_SLIDE0 | SAMPLE_SWITCH_BIT_SLIDE1 | SAMPLE_SWITCH_BIT_SLIDE2 | SAMPLE_SWITCH_BIT_SLIDE3)

/***********************************************************************************
 *  variables
 ***********************************************************************************/

/***********************************************************************************
 *  functions 
 ***********************************************************************************/
extern int cpu0_init(void);
extern uint32_t sample_detect_switch(void);

#endif /* __SAMPLE_APP_SETTING_H__ */
/***********************************************************************************
 * end of file 
 ***********************************************************************************/
